function [] = drawAnimation(raw, n, C, L, P)  
% draws the animation (including the bones, if specified)
%
% raw : data set
% n   : number of sensors in the data set
%
% C,L : relative orientations of the joints
% P   : hierarchy of the sensors ("parent relationship")

    SCALING = 2;

    [D time row] = parseData(1, raw, n);
      
    % find max/min coordinates
    minpos=ones(1,3)*1000;
    maxpos=ones(1,3)*-1000;
    while row < size(raw,1)
        [D time row] = parseData(row, raw, n);
        for i = 1:n
            minpos = min( [minpos; D(:,1:3)] );
            maxpos = max( [maxpos; D(:,1:3)] );
        end
    end
    
    b=3;
    axis([minpos(1)-b maxpos(1)+b minpos(2)-b maxpos(2)+b minpos(3)-b maxpos(3)+b]);

    grid on;
    
    drawFrame(eye(3), zeros(1,3), SCALING*5);
    
    [D time row] = parseData(1, raw, n);
    
    skeleton=zeros(n,1);
    endpoint = zeros(n,1);
    
    if nargin == 5
        deg = degree(P);
        for i = 1:n
            % draw line from i to P(i)
            if P(i) ~= 0
                posA = D(P(i), 1:3)' - frame(D(P(i),:))*(C(P(i),:)');
                posB = D(P(i), 1:3)' - frame(D(P(i),:))*(L(i,:)');
                    
                pos = [posA posB];
                skeleton(i) = line('xdata', pos(1,:), 'ydata', pos(2,:), 'zdata', pos(3,:), 'color', 'black');
            end
            
            if deg(i) == 1 % leaf node, draw a line to coordinate system
                posA = D(i, 1:3)' - frame(D(i,:))*C(i,:)';
                posB = D(i, 1:3)';
                pos = [posA posB];
                endpoint(i) = line('xdata', pos(1,:), 'ydata', pos(2,:), 'zdata', pos(3,:), 'color', 'black');
            end
        end
    end
    
    frames = zeros(n,3);
    for i = 1:n
         %frames(i,:) = drawFrame(makeRotationMatrix(D(i,4), D(i,5), D(i,6)), D(i,1:3), SCALING);
         frames(i,:) = drawFrame(frame(D(i,:)), D(i,1:3), SCALING);
    end
    
    while row < size(raw,1)
        [D time row] = parseData(row, raw, n);
        [D time row] = parseData(row, raw, n);
        for i = 1:n
            %setFrame(frames(i,:), makeRotationMatrix(D(i,6), D(i,5), D(i,4)), D(i,1:3), SCALING);
            setFrame(frames(i,:), frame(D(i,:))', D(i,1:3), SCALING);
        end
        
        if nargin == 5
            for i = 1:n
                % draw line from i to P(i)
                if P(i) ~= 0
                    %posA = D(i, 1:3)' - frame(D(i,:))*C(i,:)';
                    %posB = D(P(i), 1:3)' - frame(D(P(i),:))*C(P(i), :)';
                    posA = D(P(i), 1:3)' - frame(D(P(i),:))'*(C(P(i),:)');
                    posB = D(P(i), 1:3)' - frame(D(P(i),:))'*(L(i,:)');
                    
                    pos = [posA posB];

                    set(skeleton(i), 'xdata', pos(1,:), 'ydata', pos(2,:), 'zdata', pos(3,:));
                end
                
                if  deg(i) == 1 % leaf node, draw a line to coordinate system
                    posA = D(i, 1:3)' - frame(D(i,:))'*C(i,:)';
                    posB = D(i, 1:3)';
                    pos = [posA posB];
                    set(endpoint(i), 'xdata', pos(1,:), 'ydata', pos(2,:), 'zdata', pos(3,:));
                end
            end
        end
        
       pause(0.001);
        
    end
end


function [lines] = drawFrame(R, t, scaling)
% draws a coordinate frame

start = t'*ones(1,3);
dest = start+R*scaling;

X = [start(1,:)' dest(1,:)'];
Y = [start(2,:)' dest(2,:)'];
Z = [start(3,:)' dest(3,:)'];

lines(1) = line('xdata', X(1,:), 'ydata', Y(1,:), 'zdata', Z(1,:), 'color', 'red');
lines(2) = line('xdata', X(2,:), 'ydata', Y(2,:), 'zdata', Z(2,:), 'color', 'green');
lines(3) = line('xdata', X(3,:), 'ydata', Y(3,:), 'zdata', Z(3,:), 'color', 'blue');

end


function [lines] = setFrame(lines, R, t, scaling)
% changes a coordinate frame

    start = t'*ones(1,3);
    dest = start+R*scaling;

    X = [start(1,:)' dest(1,:)'];
    Y = [start(2,:)' dest(2,:)'];
    Z = [start(3,:)' dest(3,:)'];

    set(lines(1), 'xdata', X(1,:), 'ydata', Y(1,:), 'zdata', Z(1,:));
    set(lines(2), 'xdata', X(2,:), 'ydata', Y(2,:), 'zdata', Z(2,:));
    set(lines(3), 'xdata', X(3,:), 'ydata', Y(3,:), 'zdata', Z(3,:));
end
